/*--------------------------------------------------------------------------------*/
/* 'C' Programming Under WIN32                                    V.Burel (c)2008 */
/*                                                                                */
/*  WEB  : http://pagesperso-orange.fr/vb-audio/fr/pub/programming/index.htm      */
/*  mail : infoprog@vb-audio.com                                                  */
/*--------------------------------------------------------------------------------*/
/* This MinPrg is a minimal program under windows (like a Hello World)            */
/* To demonstrate how to make custom Windows Control                              */
/*--------------------------------------------------------------------------------*/
/*  To compile With Microsoft VC2005 you need to create an empty Win32 project    */
/*  with the following options :                                                  */
/*  - Configuration Properties / General : Char Set = NOT SET                     */
/*  - Configuration Properties / C/C++ / Preprocessor : _CRT_SECURE_NO_DEPRECATE  */
/*--------------------------------------------------------------------------------*/
#ifndef __cplusplus
	#ifndef STRICT
		#define STRICT
	#endif
#endif
#include <windows.h>
#include <stdio.h>
#include "minprg.h"
#include "tools.h"
#include "myctrl.h"

//
//define globals 
//
static HWND			G_hwnd_MainWindow	=NULL;
static HINSTANCE	G_hinstance			=NULL;

// we create a strcuture to store GDI Resources.
typedef struct tagCOLORPENBRUSH
{
	COLORREF	color;
	HPEN		pen;
	HBRUSH		brush;
} T_COLORPENBRUSH, *PT_COLORPENBRUSH, *LPT_COLORPENBRUSH;

//we create a main structure and data to store application data.
typedef struct tagAPP_CONTEXT
{
	//resources
	HBITMAP bmp_bkg;
	HBITMAP bmp_cursor;
	HBITMAP bmp_panel;

	HFONT	font_small;
	HFONT	font_med;
	HFONT	font_big;

	T_COLORPENBRUSH	gdiobjects_gray1;
	T_COLORPENBRUSH	gdiobjects_gray2;
	T_COLORPENBRUSH	gdiobjects_bleu;
	T_COLORPENBRUSH	gdiobjects_red;
	//controls:
	HWND	hw_ListBox;
	HWND	hw_EditBox;
	HWND	hw_ComboBox;
	HWND	hw_PushButton;

	HWND	hw_mybutton_1;
	HWND	hw_mybutton_2;
	HWND	hw_mybutton_3;
	HWND	hw_mybutton_4;
	HWND	hw_mybutton_5;

	HWND	hw_panel1;
	HWND	hw_panel2;

	HMENU	hPopupMenu;
	long	nbHistoryValue;
	BOOL	fUpdateFromEditBox;
	BOOL	fUpdateFromComboBox;

	UINT_PTR	IDTimer;
	//data
	long	mouse_x, mouse_y;
	float	slider_value;
	long	color_rrr;
	long	color_ggg;
	long	color_bbb;
	long	lastmouse_x;
	HWND	LastCapture;
	BOOL	fMoveSlider;
} T_APP_CONTEXT, *PT_APP_CONTEXT, *LPT_APP_CONTEXT;

static T_APP_CONTEXT G_MainAppCtx;

/*******************************************************************************/
/**                           INIT / END Resources                            **/
/*******************************************************************************/

long CreateColorPenBrush(LPT_COLORPENBRUSH lpgdi, COLORREF color)
{
	LOGBRUSH	lb;
	if (lpgdi == NULL) return -1;
	lpgdi->color=color;
	lpgdi->pen=CreatePen(PS_SOLID,0,color);

	lb.lbStyle=BS_SOLID;
	lb.lbColor=color;
	lpgdi->brush=CreateBrushIndirect(&lb);
	return 0;
}

long DestroyColorPenBrush(LPT_COLORPENBRUSH lpgdi)
{
	if (lpgdi == NULL) return -1;
	
	lpgdi->color=0;
	if (lpgdi->pen != NULL) DeleteObject(lpgdi->pen);
	lpgdi->pen=NULL;
	if (lpgdi->brush != NULL) DeleteObject(lpgdi->brush);
	lpgdi->brush=NULL;
	return 0;
}


long InitResources(LPT_APP_CONTEXT lpapp)
{
	LOGFONT		lf;

	if (lpapp == NULL) return -1;
	//Load Bitmap Resources
	lpapp->bmp_bkg		=LoadBitmap(G_hinstance,MAKEINTRESOURCE(100));
	lpapp->bmp_cursor	=LoadBitmap(G_hinstance,MAKEINTRESOURCE(101));
	lpapp->bmp_panel	=LoadBitmap(G_hinstance,MAKEINTRESOURCE(102));

	//make Font
	memset(&lf,0, sizeof(LOGFONT));
	lf.lfHeight	= 14;
	lf.lfWeight	= 400;
	strcpy(lf.lfFaceName,"Arial");
	lpapp->font_small =CreateFontIndirect(&lf);

	lf.lfHeight	= 22;
	lf.lfWeight	= 400;
	strcpy(lf.lfFaceName,"Arial");
	lpapp->font_med =CreateFontIndirect(&lf);
	
	lf.lfHeight	= 40;
	lf.lfWeight	= 400;
	strcpy(lf.lfFaceName,"Arial");
	lpapp->font_big =CreateFontIndirect(&lf);
	//make pen brush 
	CreateColorPenBrush(&(lpapp->gdiobjects_gray1), RGB(100,100,100));
	CreateColorPenBrush(&(lpapp->gdiobjects_gray2), RGB(150,150,150));
	CreateColorPenBrush(&(lpapp->gdiobjects_bleu), RGB(0,0,250));
	CreateColorPenBrush(&(lpapp->gdiobjects_red), RGB(255,0,0));
	return 0;
}

long EndResources(LPT_APP_CONTEXT lpapp)
{
	if (lpapp == NULL) return -1;
	//Delete Bitmap Object
	if (lpapp->bmp_bkg != NULL) DeleteObject(lpapp->bmp_bkg);
	lpapp->bmp_bkg=NULL;
	if (lpapp->bmp_cursor != NULL) DeleteObject(lpapp->bmp_cursor);
	lpapp->bmp_cursor=NULL;
	if (lpapp->bmp_panel != NULL) DeleteObject(lpapp->bmp_panel);
	lpapp->bmp_panel=NULL;
	//Delete font Object
	if (lpapp->font_small != NULL) DeleteObject(lpapp->font_small);
	lpapp->font_small=NULL;
	if (lpapp->font_med != NULL) DeleteObject(lpapp->font_med);
	lpapp->font_med=NULL;
	if (lpapp->font_big != NULL) DeleteObject(lpapp->font_big);
	lpapp->font_big=NULL;
	//Delete Pen Brush
	DestroyColorPenBrush(&(lpapp->gdiobjects_gray1));
	DestroyColorPenBrush(&(lpapp->gdiobjects_gray2));
	DestroyColorPenBrush(&(lpapp->gdiobjects_bleu));
	DestroyColorPenBrush(&(lpapp->gdiobjects_red));
	return 0;
}

/*******************************************************************************/
/**                                QUIT & ABOUT                               **/
/*******************************************************************************/

void ManageCloseMessage(HWND hw)
{
	int rep;
	char titre[]="Close Application...";
	char message[512];
	
	//Get the string from resource, otherwise we use a constant.
	if (LoadString(G_hinstance, IDS_CONFIRMCLOSE, message, 512) == 0)
		strcpy(message,"Do you Want To Close This Application ?\n"
						"(but there is a problem to load resources)");
	//Open System Dialog Box 
	rep=MessageBox(hw,message,titre,MB_APPLMODAL | MB_YESNO | MB_ICONQUESTION);
	if (rep == IDNO) return;

	PostMessage(hw,WM_DESTROY,0,0L); //DestroyWindow(G_hwnd_MainWindow);
}

void ManageAboutBox(HWND hw)
{
	char titre[]="About...";
	char message[512];
	strcpy(message,SZPUBLICNAME);
	strcat(message,"\nVersion : ");
	strcat(message,SZPUBLICVERSION);
	strcat(message,"\nStandalone Application\n");
	strcat(message,"\nExample of 'C' Source code\n");
	
	MessageBox(hw,message,titre,MB_APPLMODAL | MB_OK | MB_ICONINFORMATION);
}


/*******************************************************************************/
/*******************************************************************************/
/*******************************************************************************/

BOOL InitSoftware(HWND hw)
{
	InitResources(&G_MainAppCtx);
	//init some data
	G_MainAppCtx.color_rrr = 100;
	G_MainAppCtx.color_ggg = 100;
	G_MainAppCtx.color_bbb = 100;
	return TRUE;
}

BOOL EndSoftware(HWND hw)
{
	EndResources(&G_MainAppCtx);
	return TRUE;
}

/*******************************************************************************/
/*                        Display information in LCD                           */
/*******************************************************************************/

void DisplayMousePosition(LPT_APP_CONTEXT lpapp, HDC dc)
{
	char sss[64];
	long dx,dy;
	RECT rect;
	HFONT oldfont;
	//let's define some local data
	dx=190;
	dy=34;
	rect.left = 297;
	rect.top=14;
	rect.right=rect.left+dx;
	rect.bottom=rect.top+dy;
	//Draw background by using bitmap
	TOOL_DrawPartOfBitmap(dc, lpapp->bmp_bkg, rect.left,rect.top, rect.left,rect.top,dx,dy);
	//Draw X
	oldfont=(HFONT)SelectObject(dc,lpapp->font_med);
	SetTextColor(dc,lpapp->gdiobjects_gray1.color);
	SetBkMode(dc,TRANSPARENT);

	strcpy(sss,"x:");
	TextOut(dc,rect.left,rect.top+12,sss,(int)strlen(sss));

	sprintf(sss,"%i",lpapp->mouse_x);
	SelectObject(dc,lpapp->font_big);
	SetTextColor(dc,lpapp->gdiobjects_gray2.color);
	TextOut(dc,rect.left+15,rect.top,sss,(int)strlen(sss));
	//Draw Y
	rect.left +=100;
	SelectObject(dc,lpapp->font_med);
	SetTextColor(dc,lpapp->gdiobjects_gray1.color);
	strcpy(sss,"y:");
	TextOut(dc,rect.left,rect.top+12,sss,(int)strlen(sss));

	sprintf(sss,"%i",lpapp->mouse_y);
	SelectObject(dc,lpapp->font_big);
	SetTextColor(dc,lpapp->gdiobjects_gray2.color);
	TextOut(dc,rect.left+15,rect.top,sss,(int)strlen(sss));

	//replace previous GDI object in DC
	SelectObject(dc,oldfont);
}

void DisplaySlider(LPT_APP_CONTEXT lpapp, HDC dc)
{
	float fmin,fmax,fratio;
	long dx_curs;
	long dx,dy,xx;
	RECT rect;
	//let's define some local data
	dx=250;			//width of the slider
	dy=18;			//height of the slider
	rect.left =13;	//position of the slider in the Client Area
	rect.top =24;
	rect.right=rect.left+dx;
	rect.bottom=rect.top+dy;
	//compute Cursor coordinates
	dx_curs=34;
	fmin=-24.0f;
	fmax=+24.0f;
	fratio=(dx-dx_curs)/(fmax-fmin);
	xx=(long)((lpapp->slider_value-fmin)*fratio);
	if (xx<0) xx=0;
	if (xx>(dx-dx_curs)) xx=dx-dx_curs;
	//Draw Slider and Cursor in 3 steps to minimize flickering effects.
	if (xx>0) TOOL_DrawPartOfBitmap(dc, lpapp->bmp_bkg, rect.left,rect.top, rect.left,rect.top,xx,dy);
	TOOL_DrawBitmap(dc,rect.left+xx,rect.top,lpapp->bmp_cursor);
	if ((xx+dx_curs)<dx) TOOL_DrawPartOfBitmap(dc, lpapp->bmp_bkg, rect.left+xx+dx_curs,rect.top, 
												rect.left+xx+dx_curs,rect.top,dx-(xx+dx_curs),dy);

}

void DisplaySliderValueInLCD(LPT_APP_CONTEXT lpapp, HDC dc)
{
	HFONT oldfont;
	char sss[128];
	long dx,dy;
	RECT rect;
	//let's define some local data
	dx=190;
	dy=16;
	rect.left = 297;
	rect.top=52;
	rect.right=rect.left+dx;
	rect.bottom=rect.top+dy;
	//Draw background of the slider using bmp
	TOOL_DrawPartOfBitmap(dc, lpapp->bmp_bkg, rect.left,rect.top, rect.left,rect.top,dx,dy);
	//Draw value
	oldfont=(HFONT)SelectObject(dc,lpapp->font_small);
	SetTextColor(dc,lpapp->gdiobjects_red.color);
	SetBkMode(dc,TRANSPARENT);
	sprintf(sss,"Slider Value = %0.2f",lpapp->slider_value);
	DrawText(dc,sss,(int)strlen(sss),&rect,DT_SINGLELINE | DT_LEFT | DT_VCENTER);
	//replace previous GDI object in DC
	SelectObject(dc,oldfont);
}

void DisplayOurStuff(LPT_APP_CONTEXT lpapp, HDC dc)
{
	DisplayMousePosition(lpapp,dc);
	DisplaySlider(lpapp,dc);
	DisplaySliderValueInLCD(lpapp, dc);
}


void DisplayTimerStuff(LPT_APP_CONTEXT lpapp, HWND hw)
{
	RECT rect;
	HPEN oldpen;
	HBRUSH oldbrush;
	HFONT oldfont;
	HDC dc;
	char sss[64];
	SYSTEMTIME stime;
	//get system time and prepare string to display
	GetLocalTime(&stime);
	sprintf(sss,"%02i:%02i:%02i.%03i",(long)stime.wHour, (long)stime.wMinute, (long)stime.wSecond,(long)stime.wMilliseconds) ;
	dc=GetDC(hw);
	oldfont=(HFONT)SelectObject(dc,lpapp->font_big);
	oldpen=(HPEN)SelectObject(dc,GetStockObject(BLACK_PEN));
	oldbrush=(HBRUSH)SelectObject(dc,GetStockObject(WHITE_BRUSH));
	SetBkMode(dc,TRANSPARENT);
	SetTextColor(dc,RGB(lpapp->color_rrr,lpapp->color_ggg,lpapp->color_bbb));
	//display time :
	rect.left=12;
	rect.top=60;
	rect.right=266;
	rect.bottom=111;
	RoundRect(dc,rect.left,rect.top,rect.right,rect.bottom,5,5);
	DrawText(dc,sss,(int)strlen(sss),&rect,DT_SINGLELINE | DT_CENTER | DT_VCENTER);
	//deselect GDI object
	SelectObject(dc,oldpen);
	SelectObject(dc,oldbrush);
	SelectObject(dc,oldfont);
	ReleaseDC(hw,dc);
}

/*******************************************************************************/
/**                               Manage Menu                                 **/
/*******************************************************************************/

static float extractValueFromListString(char *lpchar)
{
	float ff;
	while ((*lpchar != 0) && (*lpchar != '=')) lpchar++;
	if (*lpchar != '=') return 0.0f;
	lpchar++;
	if (*lpchar == 0) return 0.0f;
	ff=(float)atof(lpchar);
	return ff;
}

void IMAIN_ProcessPopupCommand(HWND hparent, int nuCommand)
{
	char sss[1024];
	switch(nuCommand)
	{
		case IDM_CTXPOPUP_INFO:
			TOOL_GetAppDirectory(sss);
			TOOL_GetPathOnly(sss);
			ShellExecute(hparent,"open","readme.txt",NULL,sss,SW_SHOW );
			break;
		case IDM_CTXPOPUP_WEB:
			ShellExecute(hparent,"open","http://www.vb-audio.com",NULL,NULL,SW_SHOW );
			break;
		case IDM_CTXPOPUP_MAIL:
			ShellExecute(hparent,"open","mailto:vincent.burel@vb-audio.com",NULL,NULL,SW_SHOW );
			break;
	}
}


void ManageMenu(LPT_APP_CONTEXT lpapp, HWND hw, WPARAM p,LPARAM w)
{
	char sss[256];
	float ff;
	HPEN oldpen;
	HBRUSH oldbrush;
	long vi,x0,y0,nuCommand, nNotify,iSelect;
	HDC dc;
	nuCommand=(long)LOWORD(p);
	nNotify  =(long)HIWORD(p);
	switch(nuCommand)
	{
		case IDM_DRAWSOMETHING:
			dc=GetDC(hw);
			x0=300;
			y0=80;
			//Draw pixel.
			for (vi=0;vi<255;vi=vi+4)
			{
				SetPixel(dc,x0+(vi>>1),y0,RGB(vi,0,255-vi));
				SetPixel(dc,x0+1+(vi>>1),y0+1,RGB(vi,10,255-vi));
				SetPixel(dc,x0+(vi>>1),y0+2,RGB(vi,20,255-vi));
				SetPixel(dc,x0+1+(vi>>1),y0+3,RGB(vi,30,255-vi));
			}
			//Draw line.
			x0=x0+150;
			oldpen=(HPEN)SelectObject(dc,GetStockObject(BLACK_PEN));
			for (vi=0;vi<16;vi++)
			{
				MoveToEx(dc,x0+(vi<<1),y0,NULL);
				LineTo(dc,x0+(vi<<1),y0+4+vi);
			}
			//Draw empty rectangle
			x0=300;
			y0=90;
			oldbrush=SelectObject(dc,GetStockObject(NULL_BRUSH));
			Rectangle(dc,x0,y0,x0+20,y0+20);
			x0=x0+25;			
			RoundRect(dc,x0,y0,x0+20,y0+20,5,5);
			//Draw empy circle
			x0=x0+25;			
			Ellipse(dc,x0,y0,x0+20,y0+20);
			
			//Draw full rectangle
			SelectObject(dc,GetStockObject(WHITE_BRUSH));
			x0=x0+30;			
			Rectangle(dc,x0,y0,x0+20,y0+20);
			x0=x0+25;			
			RoundRect(dc,x0,y0,x0+20,y0+20,5,5);
			//Draw full circle
			x0=x0+25;			
			Ellipse(dc,x0,y0,x0+20,y0+20);
			
			//always select old objects, before releasing a DC.
			SelectObject(dc,oldpen);
			SelectObject(dc,oldbrush);
			ReleaseDC(hw,dc);
			break;
		case IDM_QUIT:
			ManageCloseMessage(hw);
			break;
		case IDM_ABOUT:
			ManageAboutBox(hw);
			break;
		//manage control
		case IDC_LISTBOX:
			if (nNotify == LBN_SELCHANGE) //something changed in the combo Box
			{
				//here the change comes from slider
				if (lpapp->fMoveSlider == TRUE) break;	
				//get Text from List Box Select Item
				iSelect=(long)SendMessage(lpapp->hw_ListBox, LB_GETCURSEL,0, 0);
				sss[0]=0;
				SendMessage(lpapp->hw_ListBox, LB_GETTEXT,(WPARAM)iSelect, (LPARAM)sss);
				ff=extractValueFromListString(sss);
				if (ff<-24.0f) ff=-24.0f;
				if (ff>24.0f) ff=24.0f;
				//let's update the Edit Box, this will update other controle
				sprintf(sss,"%0.2f",ff);
				SetWindowText(lpapp->hw_EditBox,sss);
			}
			break;
		case IDC_EDITBOX:
			if (nNotify == EN_CHANGE) //something changed in the Edit Box
			{	
				//here the value is already updated by slider
				if (lpapp->fMoveSlider == TRUE) break;	
				//here the value is already updated by combo
				if (lpapp->fUpdateFromComboBox == TRUE) break;
				//get new string in Edit Box
				GetWindowText(lpapp->hw_EditBox,sss,32);
				sss[32]=0;
				//convert to float and bind value
				ff=(float)atof(sss);
				if (ff<-24.0f) ff=-24.0f;
				if (ff>24.0f) ff=24.0f;
				//update slider value and display it
				lpapp->slider_value=ff;
				dc=GetDC(hw);
				DisplaySlider(lpapp,dc);
				DisplaySliderValueInLCD(lpapp, dc);
				ReleaseDC(hw,dc);
				//update combo Box
				lpapp->fUpdateFromEditBox=TRUE;
				if (lpapp->slider_value == -24.0f) SendMessage(lpapp->hw_ComboBox,CB_SETCURSEL ,0,0L);
				else
				{
					if (lpapp->slider_value == +24.0f) SendMessage(lpapp->hw_ComboBox,CB_SETCURSEL ,2,0L);
					else SendMessage(lpapp->hw_ComboBox,CB_SETCURSEL ,1,0L);
				}
				lpapp->fUpdateFromEditBox=FALSE;
				
			}
			break;
		case IDC_COMBOBOX:
			if (nNotify == CBN_SELCHANGE) //something changed in the combo Box
			{
				//here the value is already updated by slider
				if (lpapp->fMoveSlider == TRUE) break;	
				//here the value is already updated by Edit Box
				if (lpapp->fUpdateFromEditBox == TRUE) break;
				iSelect=(long)SendMessage(lpapp->hw_ComboBox, CB_GETCURSEL,0, 0);
				//update slider value and siplay it
				if (iSelect == 0) lpapp->slider_value=-24.0f;
				if (iSelect == 1) lpapp->slider_value=0.0f;
				if (iSelect == 2) lpapp->slider_value=+24.0f;
				//update slider value and display it
				dc=GetDC(hw);
				DisplaySlider(lpapp,dc);
				DisplaySliderValueInLCD(lpapp, dc);
				ReleaseDC(hw,dc);
				//update Edit Box
				lpapp->fUpdateFromComboBox=TRUE;
				sprintf(sss,"%0.2f",lpapp->slider_value);
				SetWindowText(lpapp->hw_EditBox,sss);
				lpapp->fUpdateFromComboBox=FALSE;
			}
			break;

		//manage our button.			
		case IDM_RESETLISTBOX:
			SendMessage(lpapp->hw_ListBox, LB_RESETCONTENT,0, 0);
			break;
		case IDM_BUTTONMIN:
			SetWindowText(lpapp->hw_EditBox,"-24.0");
			break;
		case IDM_BUTTONMID:
			SetWindowText(lpapp->hw_EditBox,"0.0");
			break;
		case IDM_BUTTONMAX:
			SetWindowText(lpapp->hw_EditBox,"24.0");
			break;

		//manage Contextual Popup
		default:
			IMAIN_ProcessPopupCommand(hw, nuCommand);
			break;

	}
}

void __stdcall My2DPanelCallback(void * lpuser,long Ident,float xvalue, float yvalue)
{
	LPT_APP_CONTEXT lpapp;
	lpapp=(LPT_APP_CONTEXT)lpuser;
	switch(Ident)
	{
	case IDC_2DPANEL_1:
		lpapp->color_bbb = (long)xvalue;
		lpapp->color_rrr = (long)yvalue;
		MYCTRL_Set2DPanelValue(lpapp->hw_panel2, (float)lpapp->color_rrr, (float)lpapp->color_ggg);
		break;
	case IDC_2DPANEL_2:
		lpapp->color_rrr = (long)xvalue;
		lpapp->color_ggg = (long)yvalue;
		MYCTRL_Set2DPanelValue(lpapp->hw_panel1, (float)lpapp->color_bbb, (float)lpapp->color_rrr);
		break;
	}
}

/*******************************************************************************/
/*                        Create Basic Window Controls                         */
/*******************************************************************************/

static long CreateOurControls(LPT_APP_CONTEXT lpapp, HWND hw)
{
	T_MYCTRL_2DPANELPARAM param_2dpanel;
	char sss[256];
	//create a list box on the right.
	lpapp->hw_ListBox=CreateWindowEx(WS_EX_CLIENTEDGE,"listbox",NULL,
						 WS_CHILD | WS_VISIBLE | ES_LEFT | WS_TABSTOP | WS_VSCROLL 
						 | LBS_NOTIFY | LBS_NOINTEGRALHEIGHT,
						 507,0,100,300,
						hw,(HMENU)IDC_LISTBOX,G_hinstance,NULL);
	if (lpapp->hw_ListBox ==NULL) return -1;
	SendMessage(lpapp->hw_ListBox,WM_SETFONT,(WPARAM)lpapp->font_small,MAKELPARAM(1,0));
	//Create an Edit Box on the bottom :
	strcpy(sss,"0.0");
	lpapp->hw_EditBox=CreateWindowEx(WS_EX_CLIENTEDGE,"edit",sss,
					 WS_CHILD | WS_VISIBLE | ES_LEFT | WS_TABSTOP,
					 10,150,100,30,
					 hw,(HMENU)IDC_EDITBOX,G_hinstance,NULL);
	if (lpapp->hw_EditBox ==NULL) return -1;
	SendMessage(lpapp->hw_EditBox,WM_SETFONT,(WPARAM)lpapp->font_med,MAKELPARAM(1,0));
	SendMessage(lpapp->hw_EditBox,EM_SETMARGINS,EC_LEFTMARGIN | EC_RIGHTMARGIN,(LPARAM) MAKELONG(2,2));
	//Create a combo box
	lpapp->hw_ComboBox=CreateWindowEx(WS_EX_CLIENTEDGE,"combobox",NULL,
						WS_VSCROLL | WS_CHILD | WS_VISIBLE | WS_TABSTOP | CBS_DROPDOWNLIST,
						120,150,150,300,
						hw,(HMENU)IDC_COMBOBOX,G_hinstance,NULL);
	if (lpapp->hw_ComboBox ==NULL) return -1;
	SendMessage(lpapp->hw_ComboBox,WM_SETFONT,(WPARAM)lpapp->font_med,MAKELPARAM(1,0));
	strcpy(sss,"Min Value");
	SendMessage(lpapp->hw_ComboBox,CB_ADDSTRING,0,(LPARAM)sss);
	strcpy(sss,"Intermediate...");
	SendMessage(lpapp->hw_ComboBox,CB_ADDSTRING,0,(LPARAM)sss);
	strcpy(sss,"Max Value");
	SendMessage(lpapp->hw_ComboBox,CB_ADDSTRING,0,(LPARAM)sss);
	SendMessage(lpapp->hw_ComboBox,CB_SETCURSEL ,1,0L);
	//Create Push Button
	lpapp->hw_PushButton=CreateWindow("button","About Box",WS_CHILD | WS_VISIBLE | WS_TABSTOP,
					 300,150,150,30,
					 hw,(HMENU)IDM_ABOUT,G_hinstance,NULL);
	if (lpapp->hw_PushButton ==NULL) return -1;
	SendMessage(lpapp->hw_PushButton,WM_SETFONT,(WPARAM)lpapp->font_med,MAKELPARAM(1,0));
	//our button
	lpapp->hw_mybutton_1=CreateWindow(MYCTRL_CLASSNAME_BUTTON,"Display About Box",WS_CHILD | WS_VISIBLE | WS_TABSTOP,
					 10,160,150,30,
					 hw,(HMENU)IDM_ABOUT,G_hinstance,NULL);

	lpapp->hw_mybutton_2=CreateWindow(MYCTRL_CLASSNAME_BUTTON,"Reset List Box",WS_CHILD | WS_VISIBLE | WS_TABSTOP,
					 10,200,150,30,
					 hw,(HMENU)IDM_RESETLISTBOX,G_hinstance,NULL);
	SendMessage(lpapp->hw_mybutton_2,WM_SETFONT,(WPARAM)lpapp->font_med,MAKELPARAM(1,0));

	//our 3 small button
	lpapp->hw_mybutton_3=CreateWindow(MYCTRL_CLASSNAME_BUTTON,"-24",WS_CHILD | WS_VISIBLE | WS_TABSTOP,
					 10,240,45,20,
					 hw,(HMENU)IDM_BUTTONMIN,G_hinstance,NULL);
	SendMessage(lpapp->hw_mybutton_3,WM_SETFONT,(WPARAM)lpapp->font_small,MAKELPARAM(1,0));
	lpapp->hw_mybutton_4=CreateWindow(MYCTRL_CLASSNAME_BUTTON,"0",WS_CHILD | WS_VISIBLE | WS_TABSTOP,
					 60,240,45,20,
					 hw,(HMENU)IDM_BUTTONMID,G_hinstance,NULL);
	SendMessage(lpapp->hw_mybutton_4,WM_SETFONT,(WPARAM)lpapp->font_small,MAKELPARAM(1,0));
	lpapp->hw_mybutton_5=CreateWindow(MYCTRL_CLASSNAME_BUTTON,"+24",WS_CHILD | WS_VISIBLE | WS_TABSTOP,
					 110,240,45,20,
					 hw,(HMENU)IDM_BUTTONMAX,G_hinstance,NULL);
	SendMessage(lpapp->hw_mybutton_5,WM_SETFONT,(WPARAM)lpapp->font_small,MAKELPARAM(1,0));
	//our 2D panel 
	TOOL_MemZERO(&param_2dpanel, sizeof(T_MYCTRL_2DPANELPARAM ));
	param_2dpanel.hbmp=lpapp->bmp_panel;
	param_2dpanel.font=lpapp->font_small;
	param_2dpanel.xmin=0.0f;
	param_2dpanel.xmax=200.0f;
	param_2dpanel.ymin=0.0f;
	param_2dpanel.ymax=+200.0f;
	param_2dpanel.lpCallback=My2DPanelCallback;
	param_2dpanel.lpuser=lpapp;
	lpapp->hw_panel1=MYCTRL_Create2DPanel(hw, IDC_2DPANEL_1, 200,160, &param_2dpanel);

	param_2dpanel.hbmp=NULL;
	lpapp->hw_panel2=MYCTRL_Create2DPanel(hw, IDC_2DPANEL_2, 320,165, &param_2dpanel);

	MYCTRL_Set2DPanelValue(lpapp->hw_panel2, (float)lpapp->color_rrr, (float)lpapp->color_ggg);
	MYCTRL_Set2DPanelValue(lpapp->hw_panel1, (float)lpapp->color_bbb, (float)lpapp->color_rrr);
	
	//Create Contextual Popup Menu
	lpapp->hPopupMenu=CreatePopupMenu();
	if (lpapp->hPopupMenu ==NULL) return -1;
	strcpy(sss,"info...");
	AppendMenu(lpapp->hPopupMenu,MF_STRING,IDM_CTXPOPUP_INFO,sss);
	strcpy(sss,"See VB's Web Pages...");
	AppendMenu(lpapp->hPopupMenu,MF_STRING,IDM_CTXPOPUP_WEB,sss);
	strcpy(sss,"E-mail me...");
	AppendMenu(lpapp->hPopupMenu,MF_STRING,IDM_CTXPOPUP_MAIL,sss);


	return 0;
}

static void DestroyOurControls(LPT_APP_CONTEXT lpapp)
{
	DestroyWindow(lpapp->hw_ListBox);
	DestroyWindow(lpapp->hw_EditBox);
	DestroyWindow(lpapp->hw_ComboBox);
	DestroyWindow(lpapp->hw_PushButton);
	DestroyMenu(lpapp->hPopupMenu);

	DestroyWindow(lpapp->hw_mybutton_1);
	DestroyWindow(lpapp->hw_mybutton_2);
	DestroyWindow(lpapp->hw_mybutton_3);
	DestroyWindow(lpapp->hw_mybutton_4);
	DestroyWindow(lpapp->hw_mybutton_5);

	DestroyWindow(lpapp->hw_panel1);
	DestroyWindow(lpapp->hw_panel2);
}
/*******************************************************************************/
/*                                  CALL BACK                                  */
/*******************************************************************************/

LRESULT CALLBACK MainWindowManageEvent(HWND hw,			//handle of the window.
											UINT msg,   //Message Ident.
											WPARAM p1,	//parameter 1.
											LPARAM p2)	//parameter 2
{
	LPT_APP_CONTEXT lpapp;
	WPARAM vv;
	POINT po;
	HPEN oldpen;
	HBRUSH oldbrush;
	HFONT oldfont;
	float ff;
	RECT rect;
	long xx,yy,bmpdx,bmpdy;
	long dx,dy,rep;
	char sss[256];
	HDC dc;
	PAINTSTRUCT ps;
	lpapp=&G_MainAppCtx;
	switch (msg)
	{


		case WM_CREATE:
			//return -1 here cancel the window creation
			if (InitSoftware(hw) == FALSE) return -1;	
			rep=CreateOurControls(lpapp, hw);
			//if there is a problem :
			if (rep != 0) 
			{
				MessageBox(NULL,"Failed to create Controls...","Application error",
										MB_APPLMODAL | MB_OK | MB_ICONERROR);
				return -1;
			}
			//create a timer 30ms
			lpapp->IDTimer=SetTimer(hw,IDT_MAINTIMER,30,NULL);
			break;
		case WM_TIMER:
			if (p1 == IDT_MAINTIMER) DisplayTimerStuff(lpapp,hw);
			return 0;
		case WM_COMMAND:
			ManageMenu(lpapp, hw,p1,p2);
			break;
		case WM_SIZE:
			//position control after window re-sizing
			dx=(long)LOWORD(p2);
			dy=(long)HIWORD(p2);
			TOOL_GetBitmapSize(lpapp->bmp_bkg,&bmpdx,&bmpdy);
			//position List
			xx=dx-bmpdx;
			if (xx<100) xx=100;
			yy=dy;
			if (yy<100) yy=100;
			SetWindowPos(lpapp->hw_ListBox,HWND_TOP,bmpdx,0, xx,yy, SWP_NOMOVE | SWP_SHOWWINDOW);
			//Position EditBox, Combo and Button on bottom
			yy=dy-40;
			if (yy<150) yy=150;
			SetWindowPos(lpapp->hw_EditBox,HWND_TOP,10,yy, 0,0, SWP_NOSIZE | SWP_SHOWWINDOW);
			SetWindowPos(lpapp->hw_ComboBox,HWND_TOP,120,yy, 0,0, SWP_NOSIZE | SWP_SHOWWINDOW);
			SetWindowPos(lpapp->hw_PushButton,HWND_TOP,300,yy, 0,0, SWP_NOSIZE | SWP_SHOWWINDOW);

			break;
		case WM_PAINT:
			dc=BeginPaint(hw,&ps);
			DisplayOurStuff(lpapp,dc);
			EndPaint(hw,&ps);
	        break;
		

		case WM_LBUTTONDOWN:
			xx=(long)LOWORD(p2);
			yy=(long)HIWORD(p2);
			//test if it's in the slider.
			rect.left =13-1;
			rect.top =24-1;
			rect.right=rect.left+250+2;
			rect.bottom=rect.top+18+2;
			if ((xx>rect.left) && (xx<rect.right) && (yy>rect.top) && (yy<rect.bottom))
			{
				lpapp->LastCapture=SetCapture(hw);
				lpapp->fMoveSlider=TRUE;
				lpapp->lastmouse_x=xx;
				break;
			}
			//otherwise display the name of the pressed key.
			sss[0]=0;
			if ((p1 & MK_CONTROL) != 0) strcat(sss,"CTRL ");
			if ((p1 & MK_SHIFT) != 0) strcat(sss,"SHIFT ");
			
			rect.left=xx;
			rect.top=yy;
			rect.right=rect.left+100;
			rect.bottom=rect.top+20;

			dc=GetDC(hw);
			oldpen=(HPEN)SelectObject(dc,GetStockObject(WHITE_PEN));			
			oldbrush=(HBRUSH)SelectObject(dc,lpapp->gdiobjects_bleu.brush);			
			Rectangle(dc,rect.left,rect.top,rect.right,rect.bottom);
			oldfont=(HFONT)SelectObject(dc,lpapp->font_med);
			SetTextColor(dc,RGB(255,255,255));
			SetBkMode(dc,TRANSPARENT);
			DrawText(dc,sss,(int)strlen(sss),&rect,DT_SINGLELINE | DT_CENTER | DT_VCENTER);
			SelectObject(dc,oldbrush);
			SelectObject(dc,oldpen);
			SelectObject(dc,oldfont);
			ReleaseDC(hw,dc);
			
			break;
		case WM_LBUTTONUP:
			if (lpapp->fMoveSlider == TRUE)
			{
				ReleaseCapture();
				if (lpapp->LastCapture != NULL) SetCapture(lpapp->LastCapture);
				lpapp->fMoveSlider=FALSE;
			}
			break;
		case WM_MOUSEMOVE:
			xx=(long)(short int)LOWORD(p2);
			yy=(long)(short int)HIWORD(p2);
			
			//display mouse position in real time
			lpapp->mouse_x=xx;
			lpapp->mouse_y=yy;
			dc=GetDC(hw);
			DisplayMousePosition(lpapp, dc);
			ReleaseDC(hw,dc);
			//check if we are moving the slider.
			if (lpapp->fMoveSlider == TRUE)
			{
				ff=(float)(xx-lpapp->lastmouse_x)*0.1f;
				if (ff != 0.0f)
				{
					lpapp->slider_value +=ff;
					if (lpapp->slider_value < -24.0f) lpapp->slider_value=-24.0f;
					if (lpapp->slider_value > 24.0f) lpapp->slider_value=24.0f;
					dc=GetDC(hw);
					DisplaySlider(lpapp, dc);
					DisplaySliderValueInLCD(lpapp, dc);
					ReleaseDC(hw,dc);
					//add Value in List Box
					lpapp->nbHistoryValue++;
					sprintf(sss,"[%04i] Slider Value = %0.3f",lpapp->nbHistoryValue, lpapp->slider_value);
					vv=(WPARAM)SendMessage(lpapp->hw_ListBox,LB_ADDSTRING,0,(LPARAM)sss);		
					SendMessage(lpapp->hw_ListBox,LB_SETCURSEL,vv,0);		
					//update Edit Box
					sprintf(sss,"%0.1f",lpapp->slider_value);
					SetWindowText(lpapp->hw_EditBox,sss);
					//update combo Box
					if (lpapp->slider_value == -24.0f) 
						SendMessage(lpapp->hw_ComboBox,CB_SETCURSEL ,0,0L);
					else
					{
						if (lpapp->slider_value == +24.0f) 
							SendMessage(lpapp->hw_ComboBox,CB_SETCURSEL ,2,0L);
						else SendMessage(lpapp->hw_ComboBox,CB_SETCURSEL ,1,0L);
					}
				}
				lpapp->lastmouse_x=xx;
			}
			break;


		case WM_RBUTTONDOWN:
			po.x=(int)(short int)LOWORD(p2);
			po.y=(int)(short int)HIWORD(p2);
			ClientToScreen(hw,&po);
			TrackPopupMenu(lpapp->hPopupMenu,0,po.x,po.y,0,hw,NULL);
			break;
		

		case WM_CLOSE:
			ManageCloseMessage(hw);
			break;
			//here we display the background of our client area.
		case WM_ERASEBKGND:
			dc=(HDC)p1;
			//display background bitmap on the top-left
			TOOL_DrawBitmap(dc, 0, 0, lpapp->bmp_bkg);
			//draw gray background below the bitmap according the client area size
			TOOL_GetBitmapSize(lpapp->bmp_bkg,&bmpdx,&bmpdy);
			GetClientRect(hw,&rect);
			dx=rect.right-rect.left;
			dy=rect.bottom-rect.top;
			if (dx>bmpdx) dx=bmpdx;
			dy=dy-bmpdy+1;
			if (dy>0)
			{
				oldpen=(HPEN)SelectObject(dc,GetStockObject(NULL_PEN));
				oldbrush=(HBRUSH)SelectObject(dc,GetStockObject(GRAY_BRUSH));
				Rectangle(dc,0,bmpdy,dx+1,bmpdy+dy);
				SelectObject(dc,oldbrush);
				SelectObject(dc,oldpen);
			}
			return 1;
		case WM_DESTROY:
			KillTimer(hw,lpapp->IDTimer);
			EndSoftware(hw);
			DestroyOurControls(lpapp);
			PostQuitMessage(0);
			break;
		default:
			return (DefWindowProc(hw,msg,p1,p2));

	}
	return (0L);
}


/*******************************************************************************/
/**                              MAIN PROCDURE                                **/
/*******************************************************************************/

int APIENTRY WinMain(HINSTANCE handle_app,			//Application hinstance.
							HINSTANCE handle_prev,  //NULL.
							LPTSTR param,           //Command Line Parameter.
							int com_show)           //How to display window (optionnal).
{
	long	wstyle;
	MSG		msg;    
	char	szWindowClassName[]="MainWindowClass";
	char *	title="Sorry";

	WNDCLASS	wc;
	//we first store the APP Hinstance
	G_hinstance=handle_app;

	//here you can make some early initialization and analyze command line if any.
	TOOL_MemZERO(&G_MainAppCtx, sizeof(T_APP_CONTEXT)); //initialize our structure with ZERO value
	TOOL_StoreOriginalAppDirectory();
	MYCTRL_InitLib(G_hinstance);
	//we define a window class to create a window from this class 
	wc.style		=CS_HREDRAW | CS_VREDRAW;  	  		//property.
	wc.lpfnWndProc=(WNDPROC)MainWindowManageEvent;		//Adresse of our Callback.
	wc.cbClsExtra =0;					  				//to store byte inside a class object.
	wc.cbWndExtra =0;                          			//to store byte inside a window object.
	wc.hInstance  =handle_app; 	                		//handle of the application hinstance.
	wc.hIcon      =LoadIcon(NULL, IDI_APPLICATION);    	//handle of icon displayed in the caption.
	wc.hCursor    =LoadCursor(NULL,IDC_ARROW);			//handle of cursor mouse .
	wc.hbrBackground=(HBRUSH)(COLOR_MENU+1);			//Background color.
	wc.lpszMenuName="MyMainAppMenu";    				//pointer on menu defined in resource.
	wc.lpszClassName=szWindowClassName;       			//pointer on class name.

	//register class.
	if (RegisterClass(&wc)==0)
	{
		MessageBox(NULL,"Failed to register main class...",title,MB_APPLMODAL | MB_OK | MB_ICONERROR);
		return 0;
	}
	//then we can create a windows from this class.
	
	//Classical Window without Sizing border.
	//wstyle=WS_DLGFRAME | WS_OVERLAPPED | WS_VISIBLE | WS_MINIMIZEBOX | WS_SYSMENU;
	
	//classical Main Window
	wstyle=WS_OVERLAPPEDWINDOW | WS_VISIBLE | WS_CLIPCHILDREN;
	G_hwnd_MainWindow=CreateWindow(szWindowClassName,	// address of registered class name.
						 SZPUBLICNAME,					// address of window name string
						 wstyle,						// window style
						 CW_USEDEFAULT,					// horizontal position of window
						 CW_USEDEFAULT,					// vertical position of window
						 UI_WIN_DX,						// window width
						 UI_WIN_DY,						// window height
						 NULL,							// parent handle is NULL.
						 NULL,							// menu name defined in resource.
						 handle_app,					// handle of application instance
						 NULL); 						// address of window-creation data

	if (G_hwnd_MainWindow==NULL)
	{
		MessageBox(NULL,"Failed to create window...",title,MB_APPLMODAL | MB_OK | MB_ICONERROR);
		return 0;
	}
	ShowWindow(G_hwnd_MainWindow,SW_SHOW);				//Display the window.
	UpdateWindow(G_hwnd_MainWindow);					//Send WM_PAINT.
	/*---------------------------------------------------------------------------*/
	/*                             Messages Loop.                                */
	/*---------------------------------------------------------------------------*/
	while (GetMessage(&msg,NULL,0,0))	//Get Message if any.
	{
		TranslateMessage(&msg);			//Translate the virtuel keys event.
		DispatchMessage(&msg);			//DispatchMessage to the related window.
	}

	//here you can make last uninitialization and release
	MYCTRL_EndLib();
	return (int)(msg.wParam);
}
