/*****************************************************************/
/* MYCTRL                                                V.Burel */
/*****************************************************************/
/* Contains 2 custom windows control                             */
/*****************************************************************/
#ifndef __cplusplus
	#ifndef STRICT
		#define STRICT
	#endif
#endif

#include <windows.h>
#include <stdio.h>
#include "myctrl.h"
#include "tools.h"

static HINSTANCE G_hinstance=NULL;

/*---------------------------------------------------------------*/
/*                        BASIC BUTTON                           */
/*---------------------------------------------------------------*/
typedef struct tagMYCTRL_BUTTONCTX
{
	HWND	hw;
	HFONT	font;
	BOOL	status;
	HWND	LastCapture;
	long	action;
} T_MYCTRL_BUTTONCTX, *PT_MYCTRL_BUTTONCTX, *LPT_MYCTRL_BUTTONCTX;

#define ACTION_BY_MOUSE 1
#define ACTION_BY_KEYBOARD 2

static void DrawMyButton(HWND hw,HDC dc)
{
	LPT_MYCTRL_BUTTONCTX lpctx;
	char sss[256]="-";
	RECT rect;
	HPEN oldpen;
	HBRUSH oldbrush;
	HFONT oldfont;
	lpctx=TOOL_RecallPointerFromWindow(hw);
	if (lpctx == NULL) return;

	
	GetClientRect(hw,&rect);
	SetBkMode(dc,TRANSPARENT);
	oldfont=(HFONT)SelectObject(dc,lpctx->font);
	if (lpctx->status == TRUE)
	{
		oldpen=(HPEN)SelectObject(dc,GetStockObject(BLACK_PEN));
		oldbrush=(HBRUSH)SelectObject(dc,GetStockObject(BLACK_BRUSH));
		SetTextColor(dc,RGB(255,255,255));
	}
	else
	{
		oldpen=(HPEN)SelectObject(dc,GetStockObject(WHITE_PEN));
		oldbrush=(HBRUSH)SelectObject(dc,GetStockObject(WHITE_BRUSH));
		SetTextColor(dc,RGB(0,0,0));
	}
	//Draw Background first
	Rectangle(dc,rect.left,rect.top,rect.right,rect.bottom);

	SelectObject(dc,GetStockObject(NULL_BRUSH));
	if (lpctx->status == FALSE) SelectObject(dc,GetStockObject(BLACK_PEN));
	else SelectObject(dc,GetStockObject(WHITE_PEN));
	RoundRect(dc,rect.left+1,rect.top+1,rect.right-1,rect.bottom-1,4,4);
	//Draw Text
	GetWindowText(hw,sss,255);
	sss[255]=0;
	DrawText(dc,sss,(int)strlen(sss),&rect,DT_SINGLELINE | DT_CENTER | DT_VCENTER);
	
	//draw focus if any
	if (GetFocus() == hw)
	{
		rect.left +=3;
		rect.top+=3;
		rect.right -=3;
		rect.bottom -=3;
		DrawFocusRect(dc,&rect);
	}
	//deselect GDI object
	SelectObject(dc,oldpen);
	SelectObject(dc,oldbrush);
	SelectObject(dc,oldfont);
}

LRESULT CALLBACK MYCTRL_ButtonCallback(HWND hw,			//handle of the window.
											UINT msg,   //Message Ident.
											WPARAM p1,	//parameter 1.
											LPARAM p2)	//parameter 2
{
	RECT rect;
	HWND hparent;
	long hwIdent,xx,yy;
	LPT_MYCTRL_BUTTONCTX lpctx;
	HDC dc;
	PAINTSTRUCT ps;
	switch (msg)
	{
		//here we create our structure
		case WM_CREATE:
			lpctx=(LPT_MYCTRL_BUTTONCTX)malloc(sizeof(T_MYCTRL_BUTTONCTX));
			if (lpctx == NULL) return -1;
			TOOL_MemZERO(lpctx,sizeof(T_MYCTRL_BUTTONCTX));
			lpctx->hw=hw;
			lpctx->font=GetStockObject(DEFAULT_GUI_FONT);
			lpctx->status = FALSE;
			TOOL_StorePointerInWindow(hw,lpctx);
			break;
		//on mouse click we get the focus.
		case WM_MOUSEACTIVATE:
			if (GetFocus()!=hw) SetFocus(hw);
			return MA_ACTIVATE;
		//we can even implement SET_FONT.
		case WM_SETFONT:
			lpctx=TOOL_RecallPointerFromWindow(hw);
			if (lpctx != NULL) 
			{
				lpctx->font = (HFONT)p1;
				if (LOWORD(p2) != 0) InvalidateRect(hw,NULL,FALSE);
			}
			break;
		//on Focus change we redraw the control
		case WM_SETFOCUS:
		case WM_KILLFOCUS:
			InvalidateRect(hw,NULL,FALSE);
			return 0;
		//here we draw all the control
		case WM_PAINT:
			dc=BeginPaint(hw,&ps);
			DrawMyButton(hw,dc);
			EndPaint(hw,&ps);
			return 0;
		//mouse behavior
		case WM_LBUTTONDOWN:
			lpctx=TOOL_RecallPointerFromWindow(hw);
			if (lpctx == NULL) break;			
			if (lpctx->action != 0) break;
			lpctx->action=ACTION_BY_MOUSE;
			lpctx->status=TRUE;
			lpctx->LastCapture=SetCapture(hw);
			dc=GetDC(hw);
			DrawMyButton(hw,dc);
			ReleaseDC(hw,dc);
			break;
		case WM_LBUTTONUP:
			lpctx=TOOL_RecallPointerFromWindow(hw);
			if (lpctx == NULL) break;			
			if (lpctx->action != ACTION_BY_MOUSE) break;
			if (lpctx->status == FALSE) break;
			xx=(long)(short int)LOWORD(p2);
			yy=(long)(short int)HIWORD(p2);
			//redraw button
			ReleaseCapture();
			if (lpctx->LastCapture != NULL) SetCapture(lpctx->LastCapture);
			lpctx->status = FALSE;
			lpctx->action = 0;
			dc=GetDC(hw);
			DrawMyButton(hw,dc);
			ReleaseDC(hw,dc);
			//notify parent if ok
			GetClientRect(hw,&rect);
			if ((xx>= rect.left) && (xx<rect.right) && (yy>=rect.top) && (yy<rect.bottom))
			{
				hparent=(HWND)GetWindowLong(hw,GWL_HWNDPARENT);
				hwIdent=(long)GetWindowLong(hw,GWL_ID);
				SendMessage(hparent, WM_COMMAND, hwIdent,0);
			}
			break;
		//Keyboard Behavior
		case WM_KEYDOWN:
			if ((p1 == VK_SPACE) || (p1 == VK_RETURN))
			{
				lpctx=TOOL_RecallPointerFromWindow(hw);
				if (lpctx == NULL) break;			
				if (lpctx->action != 0) break;
				lpctx->action=ACTION_BY_KEYBOARD;
				//redraw button
				lpctx->status = TRUE;
				dc=GetDC(hw);
				DrawMyButton(hw,dc);
				ReleaseDC(hw,dc);
				//notify parent
				hparent=(HWND)GetWindowLong(hw,GWL_HWNDPARENT);
				hwIdent=(long)GetWindowLong(hw,GWL_ID);
				SendMessage(hparent, WM_COMMAND, hwIdent,0);
			}
			return 0;
		case WM_KEYUP:
			if ((p1 == VK_SPACE) || (p1 == VK_RETURN))
			{
				lpctx=TOOL_RecallPointerFromWindow(hw);
				if (lpctx == NULL) break;			
				if (lpctx->action != ACTION_BY_KEYBOARD) break;
				lpctx->status = FALSE;
				lpctx->action = 0;
				//redraw button
				dc=GetDC(hw);
				DrawMyButton(hw,dc);
				ReleaseDC(hw,dc);
			}
			return 0;

		//close and destroy.
		case WM_CLOSE:
			DestroyWindow(hw);
			return 0;
		case WM_DESTROY:
			lpctx=TOOL_RecallPointerFromWindow(hw);
			if (lpctx != NULL) 
			{
				free(lpctx);
				TOOL_StorePointerInWindow(hw,NULL);
			}
			break;

	}
	return DefWindowProc(hw,msg,p1,p2);
}









/*---------------------------------------------------------------*/
/*                          2D PANEL                             */
/*---------------------------------------------------------------*/
typedef struct tagMYCTRL_2DPANELCTX
{
	HWND	hw;
	long	dx,dy;
	float	xvalue,yvalue;
	long	action;
	HWND	LastCapture;
	HBITMAP tempbmp;
	T_MYCTRL_2DPANELPARAM param;
} T_MYCTRL_2DPANELCTX, *PT_MYCTRL_2DPANELCTX, *LPT_MYCTRL_2DPANELCTX;

static void Draw2DPanel(LPT_MYCTRL_2DPANELCTX lpctx, HDC dc)
{
	RECT rect;
	char sss[64];
	LPT_MYCTRL_2DPANELPARAM lpp;
	HFONT oldfont;
	HPEN oldpen;
	HBRUSH oldbrush;
	HBITMAP oldbmp;
	HDC dcmem;
	float ratio;
	long xx,yy;
	long xf,yf;

	lpp=&(lpctx->param);
	//we don't draw directly in screen.
	//to avoid flickering we first build a bitmap in memory
	dcmem=CreateCompatibleDC(dc);
	if (lpctx->tempbmp ==NULL) lpctx->tempbmp=CreateCompatibleBitmap(dc,lpctx->dx,lpctx->dy);
	oldbmp=(HBITMAP)SelectObject(dcmem,lpctx->tempbmp);
	//Draw background per bitmap if any
	if (lpctx->param.hbmp != NULL)
	{
		TOOL_DrawBitmap(dcmem, 0, 0, lpctx->param.hbmp);
	}
	//Draw background per drawing function
	else
	{	
		oldpen=(HPEN)SelectObject(dcmem,GetStockObject(BLACK_PEN));
		oldbrush=(HBRUSH)SelectObject(dcmem,GetStockObject(WHITE_BRUSH));
		Rectangle(dcmem,0,0,lpctx->dx,lpctx->dy);
		SelectObject(dcmem,oldpen);
		SelectObject(dcmem,oldbrush);
	}
	//draw value intersection
	ratio=(float)(lpctx->dx-1)/(lpp->xmax-lpp->xmin);
	xx=(long)((lpctx->xvalue-lpp->xmin)*ratio);
	ratio=(float)(lpctx->dy-1)/(lpp->ymax-lpp->ymin);
	yy=(long)((lpctx->yvalue-lpp->ymin)*ratio);
	yy=lpctx->dy-yy;
	//draw vertical line
	oldpen=(HPEN)SelectObject(dcmem,GetStockObject(WHITE_PEN));
	MoveToEx(dcmem,xx,0,NULL);
	LineTo(dcmem,xx,lpctx->dy);
	SelectObject(dcmem,GetStockObject(BLACK_PEN));
	MoveToEx(dcmem,xx+1,0,NULL);
	LineTo(dcmem,xx+1,lpctx->dy);
	//draw horizontal line
	SelectObject(dcmem,GetStockObject(WHITE_PEN));
	MoveToEx(dcmem,0,yy,NULL);
	LineTo(dcmem,lpctx->dx,yy);
	SelectObject(dcmem,GetStockObject(BLACK_PEN));
	MoveToEx(dcmem,0,yy+1,NULL);
	LineTo(dcmem,lpctx->dy,yy+1);
	SelectObject(dcmem,oldpen);
	//display value
	if (lpp->font != NULL)
	{
		oldfont=(HFONT)SelectObject(dcmem,lpp->font);
		SetBkMode(dcmem,OPAQUE);
		SetTextColor(dcmem,RGB(255,255,255));
		SetBkColor(dcmem,RGB(100,100,100));
		xf=5;
		yf=5;
		if (xx<(lpctx->dx>>1)) xf=(lpctx->dx>>1)+5 ;
		if (yy<(lpctx->dy>>1)) yf=(lpctx->dy>>1)+5;
		sprintf(sss," %0.1f ",lpctx->xvalue);
		TextOut(dcmem,xf,yf,sss,(int)strlen(sss));
		sprintf(sss," %0.1f ",lpctx->yvalue);
		TextOut(dcmem,xf,yf+13,sss,(int)strlen(sss));

		SelectObject(dcmem,oldfont);
	}
	//draw focus if any
	if (GetFocus() == lpctx->hw)
	{
		GetClientRect(lpctx->hw,&rect);
		rect.left +=3;
		rect.top+=3;
		rect.right -=3;
		rect.bottom -=3;
		DrawFocusRect(dcmem,&rect);
	}
	//display bitmap on screen at the end.
	BitBlt(dc,0,0,lpctx->dx,lpctx->dy,dcmem,0,0,SRCCOPY);
	SelectObject(dcmem,oldbmp);
	DeleteDC(dcmem);
}

LRESULT CALLBACK MYCTRL_2DPanelCallback(HWND hw,		//handle of the window.
											UINT msg,   //Message Ident.
											WPARAM p1,	//parameter 1.
											LPARAM p2)	//parameter 2
{
	LPT_MYCTRL_2DPANELPARAM lpp;
	LPCREATESTRUCT   lpcs;
	LPT_MYCTRL_2DPANELCTX lpctx;
	HDC dc;
	PAINTSTRUCT ps;
	long xx,yy,hwIdent;
	float ratio;
	switch (msg)
	{
		//here we create our structure
		case WM_CREATE:
			lpcs=(LPCREATESTRUCT)p2;
			lpctx=(LPT_MYCTRL_2DPANELCTX)lpcs->lpCreateParams;
			lpctx->hw=hw;
			TOOL_StorePointerInWindow(hw,lpctx);
			break;
		//on mouse click we get the focus.
		case WM_MOUSEACTIVATE:
			if (GetFocus()!=hw) SetFocus(hw);
			return MA_ACTIVATE;
		//on Focus change we redraw the control
		case WM_SETFOCUS:
		case WM_KILLFOCUS:
			InvalidateRect(hw,NULL,FALSE);
			return 0;
		//here we draw all the control
		case WM_PAINT:
			dc=BeginPaint(hw,&ps);
			lpctx=(LPT_MYCTRL_2DPANELCTX)TOOL_RecallPointerFromWindow(hw);
			if (lpctx != NULL) Draw2DPanel(lpctx, dc);
			EndPaint(hw,&ps);
			return 0;
		//mouse behavior
		case WM_LBUTTONDOWN:
			lpctx=(LPT_MYCTRL_2DPANELCTX)TOOL_RecallPointerFromWindow(hw);
			if (lpctx == NULL) break;			
			if (lpctx->action != 0) break;
			lpctx->action=ACTION_BY_MOUSE;
			lpctx->LastCapture=SetCapture(hw);
			lpp=&(lpctx->param);
			xx=(long)(short int)LOWORD(p2);
			yy=(long)(short int)HIWORD(p2);
			//compute value according coordinates
			ratio=(float)(lpctx->dx-1)/(lpp->xmax-lpp->xmin);
			lpctx->xvalue=lpp->xmin+((float)xx/ratio);
			yy=lpctx->dy-yy;
			ratio=(float)(lpctx->dy-1)/(lpp->ymax-lpp->ymin);
			lpctx->yvalue=lpp->ymin+((float)yy/ratio);
			//display panel
			dc=GetDC(hw);
			Draw2DPanel(lpctx, dc);
			ReleaseDC(hw,dc);
			//notify to parent if any
			if (lpp->lpCallback != NULL)
			{
				hwIdent=(long)GetWindowLong(hw,GWL_ID);
				lpp->lpCallback(lpp->lpuser, hwIdent,lpctx->xvalue, lpctx->yvalue);
			}
			break;
		case WM_MOUSEMOVE:
			xx=(long)(short int)LOWORD(p2);
			yy=(long)(short int)HIWORD(p2);
			lpctx=(LPT_MYCTRL_2DPANELCTX)TOOL_RecallPointerFromWindow(hw);
			if (lpctx == NULL) break;			
			if (lpctx->action != ACTION_BY_MOUSE) break;
			lpp=&(lpctx->param);
			if (xx<0) xx=0;
			if (xx>=lpctx->dx) xx=lpctx->dx-1;
			if (yy<0) yy=0;
			if (yy>=lpctx->dy) yy=lpctx->dy-1;

			//compute value according coordinates
			ratio=(float)(lpctx->dx-1)/(lpp->xmax-lpp->xmin);
			lpctx->xvalue=lpp->xmin+((float)xx/ratio);
			yy=lpctx->dy-yy;
			ratio=(float)(lpctx->dy-1)/(lpp->ymax-lpp->ymin);
			lpctx->yvalue=lpp->ymin+((float)yy/ratio);
			//display panel
			dc=GetDC(hw);
			Draw2DPanel(lpctx, dc);
			ReleaseDC(hw,dc);
			//notify to parent if any
			if (lpp->lpCallback != NULL)
			{
				hwIdent=(long)GetWindowLong(hw,GWL_ID);
				lpp->lpCallback(lpp->lpuser, hwIdent,lpctx->xvalue, lpctx->yvalue);
			}
			break;
		case WM_LBUTTONUP:
			lpctx=(LPT_MYCTRL_2DPANELCTX)TOOL_RecallPointerFromWindow(hw);
			if (lpctx == NULL) break;			
			if (lpctx->action != ACTION_BY_MOUSE) break;
			ReleaseCapture();
			if (lpctx->LastCapture != NULL) SetCapture(lpctx->LastCapture);
			lpctx->action = 0;
			break;
		case WM_CLOSE:
			DestroyWindow(hw);
			return 0;
		case WM_DESTROY:
			lpctx=TOOL_RecallPointerFromWindow(hw);
			if (lpctx != NULL) 
			{
				if (lpctx->tempbmp !=NULL) DeleteObject(lpctx->tempbmp);
				free(lpctx);
				TOOL_StorePointerInWindow(hw,NULL);
			}
			break;

	}
	return DefWindowProc(hw,msg,p1,p2);
}

void MYCTRL_Set2DPanelValue(HWND hw, float xvalue, float yvalue)
{
	LPT_MYCTRL_2DPANELPARAM lpp;
	LPT_MYCTRL_2DPANELCTX lpctx;
	lpctx=(LPT_MYCTRL_2DPANELCTX)TOOL_RecallPointerFromWindow(hw);
	if (lpctx == NULL) return;
	lpp=&(lpctx->param);
	if (xvalue < lpp->xmin) xvalue=lpp->xmin;
	if (xvalue > lpp->xmax) xvalue=lpp->xmax;
	lpctx->xvalue=xvalue;
	if (yvalue < lpp->ymin) yvalue=lpp->ymin;
	if (yvalue > lpp->ymax) yvalue=lpp->ymax;
	lpctx->yvalue=yvalue;
	InvalidateRect(lpctx->hw,NULL,FALSE);
}

HWND MYCTRL_Create2DPanel(HWND hParent, long Ident, long x0,long y0, LPT_MYCTRL_2DPANELPARAM lpparam)
{
	HWND hw;
	long wstyle;
	LPT_MYCTRL_2DPANELCTX lpctx;
	//we allocate memory for our structure
	lpctx=(LPT_MYCTRL_2DPANELCTX)malloc(sizeof(T_MYCTRL_2DPANELCTX));
	if (lpctx == NULL) return NULL;
	TOOL_MemZERO(lpctx,sizeof(T_MYCTRL_2DPANELCTX));
	if (lpparam == NULL) return NULL;
	lpctx->param = *lpparam;
	//define control size according bitmap.
	if (lpparam->hbmp != NULL) TOOL_GetBitmapSize(lpparam->hbmp,&(lpctx->dx),&(lpctx->dy));
	else
	{
		lpctx->dx=100;
		lpctx->dy=100;
	}
	//set current values
	lpctx->xvalue=(lpctx->param.xmin+lpctx->param.xmax)*0.5f;
	lpctx->yvalue=(lpctx->param.ymin+lpctx->param.ymax)*0.5f;
	//we create window with window-creation data	
	wstyle=WS_CHILD | WS_VISIBLE | WS_TABSTOP;
	hw=CreateWindow(MYCTRL_CLASSNAME_PANEL,	NULL,
					wstyle,x0,y0,
					lpctx->dx,			// window width
					lpctx->dy,			// window height
					hParent,			// parent Windows Handle.
					(HMENU)Ident,		// child-window identifier.
					G_hinstance,		// handle of application instance
					lpctx); 			// we communicate our pointer (see WM_CREATE)

	return hw;
}


/*---------------------------------------------------------------*/
/*                           INIT / END                          */
/*---------------------------------------------------------------*/
long MYCTRL_InitLib(HINSTANCE hinst)
{
	long rep;
	WNDCLASS    wc;

	G_hinstance=hinst;
	//register class for our custom push button
	wc.style		= CS_HREDRAW | CS_VREDRAW;	 	  
	wc.lpfnWndProc	=(WNDPROC)MYCTRL_ButtonCallback;//Adresse of the related callback
	wc.cbClsExtra	=0;								//0 per default.
	wc.cbWndExtra	=sizeof(void*);					//alloc some byte to store pointer inside window.
	wc.hInstance	=hinst; 	                	//application hinstance.
	wc.hIcon		=NULL;							//no Icon
	wc.hCursor		=LoadCursor(NULL,IDC_ARROW);	//handle on cursor mouse.
	wc.hbrBackground=NULL;							//no background
	wc.lpszMenuName	=NULL;    						//no menu.
	wc.lpszClassName=MYCTRL_CLASSNAME_BUTTON;
	rep=RegisterClass(&wc);
	if (rep==0) return -1;
	
	//register class for our custom 2D Panel
	wc.style		= CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;	//allows double click 	  
	wc.lpfnWndProc	=(WNDPROC)MYCTRL_2DPanelCallback;	//Adresse of the related callback
	wc.cbClsExtra	=0;									//0 per default.
	wc.cbWndExtra	=sizeof(void*);						//alloc some byte to store pointer inside window.
	wc.hInstance	=hinst; 	                		//application hinstance.
	wc.hIcon		=NULL;								//no Icon
	wc.hCursor		=LoadCursor(NULL,IDC_UPARROW);		//handle on cursor mouse.
	wc.hbrBackground=NULL;								//no background
	wc.lpszMenuName	=NULL;    							//no menu.
	wc.lpszClassName=MYCTRL_CLASSNAME_PANEL;
	rep=RegisterClass(&wc);
	if (rep==0) return -1;
	return 0;
}

void MYCTRL_EndLib(void)
{
	UnregisterClass(MYCTRL_CLASSNAME_BUTTON,G_hinstance);
	UnregisterClass(MYCTRL_CLASSNAME_PANEL,G_hinstance);

}
